﻿using PdfSharp;
using PdfSharp.Pdf;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Web.Mvc;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.MvcEdition;

namespace Mvc_MapSuiteSample.Controllers
{
    public class HomeController : Controller
    {
        private static readonly string baseDirectory;

        static HomeController()
        {
            baseDirectory = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "App_Data");
        }

        // GET: Home
        public ActionResult Index()
        {
            return View();
        }

        [MapActionFilter]
        public void ToPdf(Map map, GeoCollection<object> args)
        {
            PdfGeoCanvas pdfGeoCanvas = new PdfGeoCanvas();

            PdfDocument document = new PdfDocument();

            PdfPage page = document.AddPage();

            page.Orientation = PageOrientation.Portrait;

            page.Size = PageSize.A4;

            // Gets the extent for print.
            RectangleShape printExtent = new RectangleShape(double.Parse(args["left"].ToString()), double.Parse(args["bottom"].ToString()), double.Parse(args["right"].ToString()), double.Parse(args["top"].ToString()));

            // Gets LayerOVerlay 
            List<LayerOverlay> lstlayeroverlay = new List<LayerOverlay>();
            lstlayeroverlay.Add((LayerOverlay)map.CustomOverlays["World"]);
            lstlayeroverlay.Add((LayerOverlay)map.CustomOverlays["ScaleBar"]);

            // Gets InMemoryMarkerOverlay
            List<InMemoryMarkerOverlay> lstmarkeroverlay = new List<InMemoryMarkerOverlay>();
            lstmarkeroverlay.Add((InMemoryMarkerOverlay)map.CustomOverlays["BoysMarkerOverlay"]);
            lstmarkeroverlay.Add((InMemoryMarkerOverlay)map.CustomOverlays["GirlsMarkerOverlay"]);


            // Prints LayerOverlay
            Collection<SimpleCandidate> labelsInLayers = new Collection<SimpleCandidate>();
            foreach (LayerOverlay overlay in lstlayeroverlay)
            {
                foreach (Layer layer in overlay.Layers)
                {
                    pdfGeoCanvas.BeginDrawing(page, printExtent, map.MapUnit);
                    layer.Open();
                    layer.Draw(pdfGeoCanvas, labelsInLayers);
                    layer.Close();
                    pdfGeoCanvas.EndDrawing();
                }
            }

            // Converts MarkerOverlay to InMemoryFeatureLayer.
            List<InMemoryFeatureLayer> markerLayers = ConvertInMemoryMarkerOverlayToInMemoryFeatureLayer(lstmarkeroverlay);

            // Prints markers
            foreach (FeatureLayer layer in markerLayers)
            {
                pdfGeoCanvas.BeginDrawing(page, printExtent, map.MapUnit);
                layer.Open();
                layer.Draw(pdfGeoCanvas, labelsInLayers);
                layer.Close();
                pdfGeoCanvas.EndDrawing();
            }

            string printingInfoFilePath = Path.Combine(baseDirectory, "Print Result.pdf");
            document.Save(printingInfoFilePath);
            Process.Start(printingInfoFilePath);
        }

        private List<InMemoryFeatureLayer> ConvertInMemoryMarkerOverlayToInMemoryFeatureLayer(List<InMemoryMarkerOverlay> lstmarkeroverlay)
        {
            List<InMemoryFeatureLayer> markerLayers = new List<InMemoryFeatureLayer>();
            foreach (InMemoryMarkerOverlay markeroverlay in lstmarkeroverlay)
            {
                // Chooses images according to markeroverlay name.
                string imagePath = Server.MapPath("~/Content/theme/default/img/marker_blue.gif");
                if (markeroverlay.Name == "BoysMarkerOverlay")
                {
                    imagePath = Server.MapPath("~/Content/theme/default/img/marker_blue.gif");
                }
                else if (markeroverlay.Name == "GirlsMarkerOverlay")
                {
                    imagePath = Server.MapPath("~/Content/theme/default/img/marker.gif");
                }

                // Creates pointStyle to shows marker as image.
                PointStyle pointStyle = new PointStyle();
                pointStyle.Image = new GeoImage(imagePath);
                pointStyle.PointType = PointType.Bitmap;

                // Let the anchor in the image match the position in the map through the offset  
                pointStyle.XOffsetInPixel = -10.5f;
                pointStyle.YOffsetInPixel = -25f;

                InMemoryFeatureLayer inMemoryFeatureLayer = new InMemoryFeatureLayer();
                inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = pointStyle;
                inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                // Add feature into inMemoryFeatureLayer.
                foreach (Feature markerFeature in markeroverlay.FeatureSource.GetAllFeatures(ReturningColumnsType.AllColumns))
                {
                    inMemoryFeatureLayer.InternalFeatures.Add(markerFeature);
                }
                markerLayers.Add(inMemoryFeatureLayer);
            }
            return markerLayers;
        }
    }
}